@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tide_corrections.py
-Written by Tyler Sutterley (03/2021)
+Written by Tyler Sutterley (06/2021)
 Calculates tidal elevations for correcting elevation or imagery data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -35,6 +35,8 @@ OPTIONS:
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate with nearest-neighbors
+    CUTOFF: Extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     FILL_VALUE: output invalid value (default NaN)
 
 PYTHON DEPENDENCIES:
@@ -50,7 +52,7 @@ PYTHON DEPENDENCIES:
 
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
-    utilities: download and management utilities for syncing files
+    utilities.py: download and management utilities for syncing files
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
@@ -67,6 +69,9 @@ PROGRAM DEPENDENCIES:
     nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 06/2021: added new Gr1km-v2 1km Greenland model from ESR
+        add try/except for input projection strings
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
     Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
         simplified netcdf inputs to be similar to binary OTIS read program
     Updated 02/2021: replaced numpy bool to prevent deprecation warning
@@ -84,7 +89,6 @@ from __future__ import print_function
 
 import os
 import pyproj
-import datetime
 import numpy as np
 import pyTMD.time
 import pyTMD.utilities
@@ -100,7 +104,7 @@ from pyTMD.read_FES_model import extract_FES_constants
 #-- PURPOSE: compute tides at points and times using tide model algorithms
 def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     EPSG=3031, EPOCH=(2000,1,1,0,0,0), TYPE='drift', TIME='UTC',
-    METHOD='spline', EXTRAPOLATE=False, FILL_VALUE=np.nan):
+    METHOD='spline', EXTRAPOLATE=False, CUTOFF=10.0, FILL_VALUE=np.nan):
     """
     Compute tides at points and times using tidal harmonics
 
@@ -130,6 +134,8 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate with nearest-neighbors
+    CUTOFF: Extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     FILL_VALUE: output invalid value (default NaN)
 
     Returns
@@ -255,6 +261,12 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         model_format = 'OTIS'
         model_EPSG = 'PSNorth'
         model_type = 'z'
+    elif (MODEL == 'Gr1km-v2'):
+        grid_file = os.path.join(DIRECTORY,'greenlandTMD_v2','grid_Greenland8.v2')
+        model_file = os.path.join(DIRECTORY,'greenlandTMD_v2','h_Greenland8.v2')
+        model_format = 'OTIS'
+        model_EPSG = '3413'
+        model_type = 'z'
     elif (MODEL == 'GOT4.7'):
         model_directory = os.path.join(DIRECTORY,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
@@ -344,7 +356,12 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         raise Exception("Unlisted tide model")
 
     #-- converting x,y from EPSG to latitude/longitude
-    crs1 = pyproj.CRS.from_string("epsg:{0:d}".format(EPSG))
+    try:
+        #-- EPSG projection code string or int
+        crs1 = pyproj.CRS.from_string("epsg:{0:d}".format(int(EPSG)))
+    except (ValueError,pyproj.exceptions.CRSError):
+        #-- Projection SRS string
+        crs1 = pyproj.CRS.from_string(EPSG)
     crs2 = pyproj.CRS.from_string("epsg:{0:d}".format(4326))
     transformer = pyproj.Transformer.from_crs(crs1, crs2, always_xy=True)
     lon,lat = transformer.transform(x.flatten(), y.flatten())
@@ -382,22 +399,22 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(lon, lat, grid_file, model_file,
             model_EPSG, TYPE=model_type, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
-            GRID=model_format)
+            CUTOFF=CUTOFF, GRID=model_format)
         deltat = np.zeros_like(t)
     elif (model_format == 'netcdf'):
         amp,ph,D,c = extract_netcdf_constants(lon, lat, grid_file, model_file,
             TYPE=model_type, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
-            SCALE=SCALE, GZIP=GZIP)
+            CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
         deltat = np.zeros_like(t)
     elif (model_format == 'GOT'):
         amp,ph,c = extract_GOT_constants(lon, lat, model_file, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file, t)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(lon, lat, model_file, TYPE=model_type,
             VERSION=MODEL, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
-            SCALE=SCALE, GZIP=GZIP)
+            CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file, t)
 
