@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_netcdf_model.py (03/2021)
+read_netcdf_model.py (06/2021)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -29,6 +29,8 @@ OPTIONS:
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate model using nearest-neighbors
+    CUTOFF: extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -52,6 +54,8 @@ PROGRAM DEPENDENCIES:
     nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 06/2021: add warning for tide models being entered as string
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
     Updated 03/2021: add extrapolation check where there are no invalid points
         prevent ComplexWarning for fill values when calculating amplitudes
         simplified inputs to be similar to binary OTIS read program
@@ -73,6 +77,7 @@ import os
 import gzip
 import uuid
 import netCDF4
+import warnings
 import numpy as np
 import scipy.interpolate
 from pyTMD.bilinear_interp import bilinear_interp
@@ -80,7 +85,7 @@ from pyTMD.nearest_extrap import nearest_extrap
 
 #-- PURPOSE: extract tidal harmonic constants from tide models at coordinates
 def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
-    METHOD='spline', EXTRAPOLATE=False, GZIP=True, SCALE=1.0):
+    METHOD='spline', EXTRAPOLATE=False, CUTOFF=10.0, GZIP=True, SCALE=1.0):
     """
     Reads files for a netCDF4 tidal model
     Makes initial calculations to run the tide program
@@ -106,6 +111,8 @@ def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate model using nearest-neighbors
+    CUTOFF: extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -117,6 +124,11 @@ def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
     constituents: list of model constituents
     """
 
+    #-- raise warning if model files are entered as a string
+    if isinstance(model_files,str):
+        warnings.warn("Tide model is entered as a string")
+        model_files = [model_files]
+
     #-- read the tide grid file for bathymetry and spatial coordinates
     lon,lat,bathymetry = read_netcdf_grid(grid_file, TYPE, GZIP=GZIP)
     #-- grid step size of tide model
@@ -226,9 +238,9 @@ def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
                 inv, = np.nonzero(z1.mask)
                 #-- replace invalid values with nan
                 z[z.mask] = np.nan
-                #-- extrapolate points within 10km of valid model points
+                #-- extrapolate points within cutoff of valid model points
                 z1.data[inv] = nearest_extrap(lon,lat,z,ilon[inv],ilat[inv],
-                    dtype=z.dtype,cutoff=10.0)
+                    dtype=z.dtype,cutoff=CUTOFF)
                 #-- replace nan values with fill_value
                 z1.mask[inv] = np.isnan(z1.data[inv])
                 z1.data[z1.mask] = z1.fill_value
@@ -277,9 +289,9 @@ def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
                 inv, = np.nonzero(tr1.mask)
                 #-- replace invalid values with nan
                 tr[tr.mask] = np.nan
-                #-- extrapolate points within 10km of valid model points
+                #-- extrapolate points within cutoff of valid model points
                 tr1.data[inv] = nearest_extrap(lon,lat,tr,ilon[inv],ilat[inv],
-                    dtype=tr.dtype,cutoff=10.0)
+                    dtype=tr.dtype,cutoff=CUTOFF)
                 #-- replace nan values with fill_value
                 tr1.mask[inv] = np.isnan(tr1.data[inv])
                 tr1.data[tr1.mask] = tr1.fill_value
