@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_tide_model.py (03/2021)
+read_tide_model.py (06/2021)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -29,6 +29,8 @@ OPTIONS:
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate model using nearest-neighbors
+    CUTOFF: extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -52,6 +54,9 @@ PROGRAM DEPENDENCIES:
     nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 06/2021: fix tidal currents for bilinear interpolation
+        check for nan points when reading elevation and transport files
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
     Updated 03/2021: add extrapolation check where there are no invalid points
         prevent ComplexWarning for fill values when calculating amplitudes
         can read from single constituent TPXO9 ATLAS binary files
@@ -87,7 +92,7 @@ import matplotlib.pyplot as plt
 
 #-- PURPOSE: extract tidal harmonic constants from tide models at coordinates
 def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
-    METHOD='spline', EXTRAPOLATE=False, GRID='OTIS'):
+    METHOD='spline', EXTRAPOLATE=False, CUTOFF=10.0, GRID='OTIS'):
     """
     Reads files for an OTIS-formatted tidal model
     Makes initial calculations to run the tide program
@@ -114,6 +119,8 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
     EXTRAPOLATE: extrapolate model using nearest-neighbors
+    CUTOFF: extrapolation cutoff in kilometers
+        set to np.inf to extrapolate for all points
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -285,9 +292,9 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 inv, = np.nonzero(z1.mask)
                 #-- replace zero values with nan
                 z[z==0] = np.nan
-                #-- extrapolate points within 10km of valid model points
+                #-- extrapolate points within cutoff of valid model points
                 z1.data[inv] = nearest_extrap(xi,yi,z,x[inv],y[inv],
-                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                    dtype=np.complex128,cutoff=CUTOFF,EPSG=EPSG)
                 #-- replace nan values with fill_value
                 z1.mask[inv] = np.isnan(z1.data[inv])
                 z1.data[z1.mask] = z1.fill_value
@@ -344,9 +351,9 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 inv, = np.nonzero(u1.mask)
                 #-- replace zero values with nan
                 u[u==0] = np.nan
-                #-- extrapolate points within 10km of valid model points
+                #-- extrapolate points within cutoff of valid model points
                 u1.data[inv] = nearest_extrap(xu,yi,u,x[inv],y[inv],
-                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                    dtype=np.complex128,cutoff=CUTOFF,EPSG=EPSG)
                 #-- replace nan values with fill_value
                 u1.mask[inv] = np.isnan(u1.data[inv])
                 u1.data[u1.mask] = u1.fill_value
@@ -376,7 +383,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 #-- replace zero values with nan
                 v[v==0] = np.nan
                 #-- use quick bilinear to interpolate values
-                v1.data = bilinear_interp(xi,yv,v,x,y,dtype=np.complex128)
+                v1.data[:] = bilinear_interp(xi,yv,v,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
                 v1.mask = (np.isnan(v1.data) | (~mv1.astype(bool)))
                 v1.data[v1.mask] = v1.fill_value
@@ -404,9 +411,9 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 inv, = np.nonzero(v1.mask)
                 #-- replace zero values with nan
                 v[z==v] = np.nan
-                #-- extrapolate points within 10km of valid model points
+                #-- extrapolate points within cutoff of valid model points
                 v1.data[inv] = nearest_extrap(x,yv,v,x[inv],y[inv],
-                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                    dtype=np.complex128,cutoff=CUTOFF,EPSG=EPSG)
                 #-- replace nan values with fill_value
                 v1.mask[inv] = np.isnan(v1.data[inv])
                 v1.data[v1.mask] = v1.fill_value
@@ -673,6 +680,10 @@ def read_elevation_file(input_file,ic):
         temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nx)
         h.data.real[i,:] = temp[0:2*nx-1:2]
         h.data.imag[i,:] = temp[1:2*nx:2]
+    #-- update mask for nan values
+    h.mask[np.isnan(h.data)] = True
+    #-- replace masked values with fill value
+    h.data[h.mask] = h.fill_value
     #-- close the file
     fid.close()
     #-- return the elevation
@@ -806,6 +817,12 @@ def read_transport_file(input_file,ic):
         u.data.imag[i,:] = temp[1:4*nx-2:4]
         v.data.real[i,:] = temp[2:4*nx-1:4]
         v.data.imag[i,:] = temp[3:4*nx:4]
+    #-- update mask for nan values
+    u.mask[np.isnan(u.data)] = True
+    v.mask[np.isnan(v.data)] = True
+    #-- replace masked values with fill value
+    u.data[u.mask] = u.fill_value
+    v.data[v.mask] = v.fill_value
     #-- close the file
     fid.close()
     #-- return the transport components
