@@ -1,12 +1,18 @@
-"""
+#!/usr/bin/env python
+u"""
 utilities.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Download and management utilities for syncing time and auxiliary files
 
 PYTHON DEPENDENCIES:
     lxml: processing XML and HTML in Python (https://pypi.python.org/pypi/lxml)
 
 UPDATE HISTORY:
+    Updated 03/2021: added sha1 option for retrieving file hashes
+    Updated 01/2021: added username and password to ftp functions
+        added ftp connection check
+    Updated 12/2020: added file object keyword for downloads if verbose
+        add url split function for creating url location lists
     Updated 11/2020: normalize source and destination paths in copy
         make context an optional keyword argument in from_http
     Updated 09/2020: copy from http and https to bytesIO object in chunks
@@ -15,7 +21,7 @@ UPDATE HISTORY:
     Updated 08/2020: add GSFC CDDIS opener, login and download functions
     Written 08/2020
 """
-from __future__ import print_function
+from __future__ import print_function, division
 
 import sys
 import os
@@ -58,24 +64,55 @@ def get_data_path(relpath):
     elif isinstance(relpath,str):
         return os.path.join(filepath,relpath)
 
-#-- PURPOSE: get the MD5 hash value of a file
-def get_hash(local):
+#-- PURPOSE: get the hash value of a file
+def get_hash(local, algorithm='MD5'):
     """
-    Get the MD5 hash value from a local file
+    Get the hash value from a local file or BytesIO object
 
     Arguments
     ---------
-    local: path to file
+    local: BytesIO object or path to file
+
+    Keyword Arguments
+    -----------------
+    algorithm: hashing algorithm for checksum validation
+        MD5: Message Digest
+        sha1: Secure Hash Algorithm
     """
-    #-- check if local file exists
-    if os.access(os.path.expanduser(local),os.F_OK):
+    #-- check if open file object or if local file exists
+    if isinstance(local, io.IOBase):
+        if (algorithm == 'MD5'):
+            return hashlib.md5(local.getvalue()).hexdigest()
+        elif (algorithm == 'sha1'):
+            return hashlib.sha1(local.getvalue()).hexdigest()
+    elif os.access(os.path.expanduser(local),os.F_OK):
         #-- generate checksum hash for local file
         #-- open the local_file in binary read mode
         with open(os.path.expanduser(local), 'rb') as local_buffer:
-            return hashlib.md5(local_buffer.read()).hexdigest()
+            #-- generate checksum hash for a given type
+            if (algorithm == 'MD5'):
+                return hashlib.md5(local_buffer.read()).hexdigest()
+            elif (algorithm == 'sha1'):
+                return hashlib.sha1(local_buffer.read()).hexdigest()
     else:
         return ''
 
+#-- PURPOSE: recursively split a url path
+def url_split(s):
+    """
+    Recursively split a url path into a list
+
+    Arguments
+    ---------
+    s: url string
+    """
+    head, tail = posixpath.split(s)
+    if head in ('http:','https:'):
+        return s,
+    elif head in ('', posixpath.sep):
+        return tail,
+    return url_split(head) + (tail,)
+
 #-- PURPOSE: convert Roman numerals to (Arabic) integers
 def roman_to_int(roman):
     """
@@ -87,10 +124,12 @@ def roman_to_int(roman):
     """
     #-- mapping between Roman and Arabic numerals
     roman_map = {'i':1, 'v':5, 'x':10, 'l':50, 'c':100, 'd':500, 'm':1000}
+    #-- verify case
+    roman = roman.lower()
     output = 0
     #-- iterate through roman numerals in string and calculate total
     for i,s in enumerate(roman):
-        if i > 0 and roman_map[s] > roman_map[roman[i-1]]:
+        if (i > 0) and (roman_map[s] > roman_map[roman[i-1]]):
             output += roman_map[s] - 2*roman_map[roman[i-1]]
         else:
             output += roman_map[s]
@@ -117,6 +156,17 @@ def get_unix_time(time_string, format='%Y-%m-%d %H:%M:%S'):
     else:
         return calendar.timegm(parsed_time)
 
+#-- PURPOSE: rounds a number to an even number less than or equal to original
+def even(value):
+    """
+    Rounds a number to an even number less than or equal to original
+
+    Arguments
+    ---------
+    value: number to be rounded
+    """
+    return 2*int(value//2)
+
 #-- PURPOSE: make a copy of a file with all system information
 def copy(source, destination, verbose=False, move=False):
     """
@@ -140,8 +190,35 @@ def copy(source, destination, verbose=False, move=False):
     if move:
         os.remove(source)
 
+#-- PURPOSE: check ftp connection
+def check_ftp_connection(HOST,username=None,password=None):
+    """
+    Check internet connection with ftp host
+
+    Arguments
+    ---------
+    HOST: remote ftp host
+
+    Keyword arguments
+    -----------------
+    username: ftp username
+    password: ftp password
+    """
+    #-- attempt to connect to ftp host
+    try:
+        f = ftplib.FTP(HOST)
+        f.login(username, password)
+        f.voidcmd("NOOP")
+    except IOError:
+        raise RuntimeError('Check internet connection')
+    except ftplib.error_perm:
+        raise RuntimeError('Check login credentials')
+    else:
+        return True
+
 #-- PURPOSE: list a directory on a ftp host
-def ftp_list(HOST,timeout=None,basename=False,pattern=None,sort=False):
+def ftp_list(HOST,username=None,password=None,timeout=None,
+    basename=False,pattern=None,sort=False):
     """
     List a directory on a ftp host
 
@@ -151,6 +228,8 @@ def ftp_list(HOST,timeout=None,basename=False,pattern=None,sort=False):
 
     Keyword arguments
     -----------------
+    username: ftp username
+    password: ftp password
     timeout: timeout in seconds for blocking operations
     basename: return the file or directory basename instead of the full path
     pattern: regular expression pattern for reducing list
@@ -167,7 +246,7 @@ def ftp_list(HOST,timeout=None,basename=False,pattern=None,sort=False):
     except (socket.gaierror,IOError):
         raise RuntimeError('Unable to connect to {0}'.format(HOST[0]))
     else:
-        ftp.login()
+        ftp.login(username,password)
         #-- list remote path
         output = ftp.nlst(posixpath.join(*HOST[1:]))
         #-- get last modified date of ftp files and convert into unix time
@@ -204,8 +283,8 @@ def ftp_list(HOST,timeout=None,basename=False,pattern=None,sort=False):
         return (output,mtimes)
 
 #-- PURPOSE: download a file from a ftp host
-def from_ftp(HOST,timeout=None,local=None,hash='',chunk=16384,
-    verbose=False,mode=0o775):
+def from_ftp(HOST,username=None,password=None,timeout=None,local=None,
+    hash='',chunk=8192,verbose=False,fid=sys.stdout,mode=0o775):
     """
     Download a file from a ftp host
 
@@ -215,11 +294,14 @@ def from_ftp(HOST,timeout=None,local=None,hash='',chunk=16384,
 
     Keyword arguments
     -----------------
+    username: ftp username
+    password: ftp password
     timeout: timeout in seconds for blocking operations
     local: path to local file
     hash: MD5 hash of local file
     chunk: chunk size for transfer encoding
     verbose: print file transfer information
+    fid: open file object to print if verbose
     mode: permissions mode of output local file
 
     Returns
@@ -233,28 +315,40 @@ def from_ftp(HOST,timeout=None,local=None,hash='',chunk=16384,
     except (socket.gaierror,IOError):
         raise RuntimeError('Unable to connect to {0}'.format(HOST[0]))
     else:
-        ftp.login()
+        ftp.login(username,password)
         #-- remote path
         ftp_remote_path = posixpath.join(*HOST[1:])
         #-- copy remote file contents to bytesIO object
         remote_buffer = io.BytesIO()
-        ftp.retrbinary('RETR {0}'.format(ftp_remote_path), remote_buffer.write)
+        ftp.retrbinary('RETR {0}'.format(ftp_remote_path),
+            remote_buffer.write, blocksize=chunk)
         remote_buffer.seek(0)
         #-- save file basename with bytesIO object
         remote_buffer.filename = HOST[-1]
         #-- generate checksum hash for remote file
         remote_hash = hashlib.md5(remote_buffer.getvalue()).hexdigest()
+        #-- get last modified date of remote file and convert into unix time
+        mdtm = ftp.sendcmd('MDTM {0}'.format(ftp_remote_path))
+        remote_mtime = get_unix_time(mdtm[4:], format="%Y%m%d%H%M%S")
         #-- compare checksums
         if local and (hash != remote_hash):
+            #-- convert to absolute path
+            local = os.path.abspath(local)
+            #-- create directory if non-existent
+            if not os.access(os.path.dirname(local), os.F_OK):
+                os.makedirs(os.path.dirname(local), mode)
             #-- print file information
             if verbose:
-                print('{0} -->\n\t{1}'.format(posixpath.join(*HOST),local))
+                args = (posixpath.join(*HOST),local)
+                print('{0} -->\n\t{1}'.format(*args), file=fid)
             #-- store bytes to file using chunked transfer encoding
             remote_buffer.seek(0)
             with open(os.path.expanduser(local), 'wb') as f:
                 shutil.copyfileobj(remote_buffer, f, chunk)
             #-- change the permissions mode
             os.chmod(local,mode)
+            #-- keep remote modification time of file and local access time
+            os.utime(local, (os.stat(local).st_atime, remote_mtime))
         #-- close the ftp connection
         ftp.close()
         #-- return the bytesIO object
@@ -264,13 +358,13 @@ def from_ftp(HOST,timeout=None,local=None,hash='',chunk=16384,
 #-- PURPOSE: check internet connection
 def check_connection(HOST):
     """
-    Check internet connection
+    Check internet connection with http host
 
     Arguments
     ---------
     HOST: remote http host
     """
-    #-- attempt to connect to https host
+    #-- attempt to connect to http host
     try:
         urllib2.urlopen(HOST,timeout=20,context=ssl.SSLContext())
     except urllib2.URLError:
@@ -280,7 +374,7 @@ def check_connection(HOST):
 
 #-- PURPOSE: download a file from a http host
 def from_http(HOST,timeout=None,context=ssl.SSLContext(),local=None,hash='',
-    chunk=16384,verbose=False,mode=0o775):
+    chunk=16384,verbose=False,fid=sys.stdout,mode=0o775):
     """
     Download a file from a http host
 
@@ -296,6 +390,7 @@ def from_http(HOST,timeout=None,context=ssl.SSLContext(),local=None,hash='',
     hash: MD5 hash of local file
     chunk: chunk size for transfer encoding
     verbose: print file transfer information
+    fid: open file object to print if verbose
     mode: permissions mode of output local file
 
     Returns
@@ -320,9 +415,15 @@ def from_http(HOST,timeout=None,context=ssl.SSLContext(),local=None,hash='',
         remote_hash = hashlib.md5(remote_buffer.getvalue()).hexdigest()
         #-- compare checksums
         if local and (hash != remote_hash):
+            #-- convert to absolute path
+            local = os.path.abspath(local)
+            #-- create directory if non-existent
+            if not os.access(os.path.dirname(local), os.F_OK):
+                os.makedirs(os.path.dirname(local), mode)
             #-- print file information
             if verbose:
-                print('{0} -->\n\t{1}'.format(posixpath.join(*HOST),local))
+                args = (posixpath.join(*HOST),local)
+                print('{0} -->\n\t{1}'.format(*args), file=fid)
             #-- store bytes to file using chunked transfer encoding
             remote_buffer.seek(0)
             with open(os.path.expanduser(local), 'wb') as f:
@@ -476,7 +577,7 @@ def cddis_list(HOST,username=None,password=None,build=True,timeout=None,
 
 #-- PURPOSE: download a file from a GSFC CDDIS https server
 def from_cddis(HOST,username=None,password=None,build=True,timeout=None,
-    local=None,hash='',chunk=16384,verbose=False,mode=0o775):
+    local=None,hash='',chunk=16384,verbose=False,fid=sys.stdout,mode=0o775):
     """
     Download a file from GSFC CDDIS archive server
 
@@ -494,6 +595,7 @@ def from_cddis(HOST,username=None,password=None,build=True,timeout=None,
     hash: MD5 hash of local file
     chunk: chunk size for transfer encoding
     verbose: print file transfer information
+    fid: open file object to print if verbose
     mode: permissions mode of output local file
 
     Returns
@@ -532,9 +634,15 @@ def from_cddis(HOST,username=None,password=None,build=True,timeout=None,
         remote_hash = hashlib.md5(remote_buffer.getvalue()).hexdigest()
         #-- compare checksums
         if local and (hash != remote_hash):
+            #-- convert to absolute path
+            local = os.path.abspath(local)
+            #-- create directory if non-existent
+            if not os.access(os.path.dirname(local), os.F_OK):
+                os.makedirs(os.path.dirname(local), mode)
             #-- print file information
             if verbose:
-                print('{0} -->\n\t{1}'.format(posixpath.join(*HOST),local))
+                args = (posixpath.join(*HOST),local)
+                print('{0} -->\n\t{1}'.format(*args), file=fid)
             #-- store bytes to file using chunked transfer encoding
             remote_buffer.seek(0)
             with open(os.path.expanduser(local), 'wb') as f:
