@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tidal_elevations.py
-Written by Tyler Sutterley (03/2021)
+Written by Tyler Sutterley (06/2021)
 Calculates tidal elevations for an input file
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -34,6 +34,7 @@ COMMAND LINE OPTIONS:
         AODTM-5
         AOTIM-5
         AOTIM-5-2018
+        Gr1km-v2
         GOT4.7
         GOT4.7_load
         GOT4.8
@@ -66,6 +67,8 @@ COMMAND LINE OPTIONS:
         nearest
         bilinear
     -E X, --extrapolate X: Extrapolate with nearest-neighbors
+    -c X, --cutoff X: Extrapolation cutoff in kilometers
+        set to inf to extrapolate for all points
     -V, --verbose: Verbose output of processing run
     -M X, --mode X: Permission mode of output file
 
@@ -89,7 +92,7 @@ PYTHON DEPENDENCIES:
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
     spatial.py: utilities for reading and writing spatial data
-    utilities: download and management utilities for syncing files
+    utilities.py: download and management utilities for syncing files
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
@@ -105,6 +108,8 @@ PROGRAM DEPENDENCIES:
     predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 06/2021: added new Gr1km-v2 1km Greenland model from ESR
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
     Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
         simplified netcdf inputs to be similar to binary OTIS read program
     Updated 02/2021: replaced numpy bool to prevent deprecation warning
@@ -148,7 +153,7 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
     TIDE_MODEL=None, FORMAT='csv', VARIABLES=['time','lat','lon','data'],
     HEADER=0, TYPE='drift', TIME_UNITS='days since 1858-11-17T00:00:00',
     TIME=None, PROJECTION='4326', METHOD='spline', EXTRAPOLATE=False,
-    VERBOSE=False, MODE=0o775):
+    CUTOFF=None, VERBOSE=False, MODE=0o775):
 
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
@@ -315,6 +320,15 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         model_type = 'z'
+    elif (TIDE_MODEL == 'Gr1km-v2'):
+        grid_file = os.path.join(tide_dir,'greenlandTMD_v2','grid_Greenland8.v2')
+        model_file = os.path.join(tide_dir,'greenlandTMD_v2','h_Greenland8.v2')
+        reference = 'https://www.esr.org/research/polar-tide-models/'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
+        model_format = 'OTIS'
+        EPSG = '3413'
+        model_type = 'z'
     elif (TIDE_MODEL == 'GOT4.7'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
@@ -484,8 +498,10 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
     #-- converting x,y from projection to latitude/longitude
     #-- could try to extract projection attributes from netCDF4 and HDF5 files
     try:
+        #-- EPSG projection code string or int
         crs1 = pyproj.CRS.from_string("epsg:{0:d}".format(int(PROJECTION)))
     except (ValueError,pyproj.exceptions.CRSError):
+        #-- Projection SRS string
         crs1 = pyproj.CRS.from_string(PROJECTION)
     crs2 = pyproj.CRS.from_string("epsg:{0:d}".format(4326))
     transformer = pyproj.Transformer.from_crs(crs1, crs2, always_xy=True)
@@ -515,23 +531,25 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(lon.flatten(), lat.flatten(),
             grid_file, model_file, EPSG, TYPE=model_type, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, GRID=model_format)
         deltat = np.zeros((nt))
     elif (model_format == 'netcdf'):
         amp,ph,D,c = extract_netcdf_constants(lon.flatten(), lat.flatten(),
             grid_file, model_file, TYPE=model_type, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=model_scale,
+            GZIP=GZIP)
         deltat = np.zeros((nt))
     elif (model_format == 'GOT'):
         amp,ph,c = extract_GOT_constants(lon.flatten(), lat.flatten(),
             model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
-            SCALE=model_scale, GZIP=GZIP)
+            CUTOFF=CUTOFF, SCALE=model_scale, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file,tide_time)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(lon.flatten(), lat.flatten(),
             model_file, TYPE=model_type, VERSION=TIDE_MODEL, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=model_scale,
+            GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file,tide_time)
 
@@ -602,7 +620,7 @@ def main():
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
         'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
-        'AODTM-5','AOTIM-5','AOTIM-5-2018',
+        'AODTM-5','AOTIM-5','AOTIM-5-2018','Gr1km-v2',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
         'FES2014','FES2014_load')
     parser.add_argument('--tide','-T',
@@ -650,6 +668,11 @@ def main():
     parser.add_argument('--extrapolate','-E',
         default=False, action='store_true',
         help='Extrapolate with nearest-neighbors')
+    #-- extrapolation cutoff in kilometers
+    #-- set to inf to extrapolate over all points
+    parser.add_argument('--cutoff','-c',
+        type=np.float64, default=10.0,
+        help='Extrapolation cutoff in kilometers')
     #-- verbose output of processing run
     #-- print information about each input and output file
     parser.add_argument('--verbose','-V',
@@ -673,7 +696,7 @@ def main():
         HEADER=args.header, TYPE=args.type, TIME_UNITS=args.epoch,
         TIME=args.deltatime, PROJECTION=args.projection,
         METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
-        VERBOSE=args.verbose, MODE=args.mode)
+        CUTOFF=args.cutoff, VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
