@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tides_ICESat2_ATL06.py
-Written by Tyler Sutterley (04/2021)
+Written by Tyler Sutterley (06/2021)
 Calculates tidal elevations for correcting ICESat-2 land ice elevation data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -28,6 +28,7 @@ COMMAND LINE OPTIONS:
         AODTM-5
         AOTIM-5
         AOTIM-5-2018
+        Gr1km-v2
         GOT4.7
         GOT4.7_load
         GOT4.8
@@ -42,6 +43,8 @@ COMMAND LINE OPTIONS:
         nearest
         bilinear
     -E X, --extrapolate X: Extrapolate with nearest-neighbors
+    -c X, --cutoff X: Extrapolation cutoff in kilometers
+        set to inf to extrapolate for all points
     -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
@@ -59,7 +62,7 @@ PYTHON DEPENDENCIES:
 PROGRAM DEPENDENCIES:
     read_ICESat2_ATL06.py: reads ICESat-2 land ice along-track height data files
     time.py: utilities for calculating time operations
-    utilities: download and management utilities for syncing files
+    utilities.py: download and management utilities for syncing files
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
@@ -75,6 +78,8 @@ PROGRAM DEPENDENCIES:
     predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 06/2021: added new Gr1km-v2 1km Greenland model from ESR
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
     Updated 04/2021: can use a generically named ATL06 file as input
     Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
         simplified netcdf inputs to be similar to binary OTIS read program
@@ -124,7 +129,7 @@ from icesat2_toolkit.read_ICESat2_ATL06 import read_HDF5_ATL06
 #-- PURPOSE: read ICESat-2 land ice data (ATL06) from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
 def compute_tides_ICESat2(tide_dir, INPUT_FILE, TIDE_MODEL=None, METHOD='spline',
-    EXTRAPOLATE=False, VERBOSE=False, MODE=0o775):
+    EXTRAPOLATE=False, CUTOFF=None, VERBOSE=False, MODE=0o775):
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
         grid_file = os.path.join(tide_dir,'cats0201_tmd','grid_CATS')
@@ -328,6 +333,18 @@ def compute_tides_ICESat2(tide_dir, INPUT_FILE, TIDE_MODEL=None, METHOD='spline'
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         TYPE = 'z'
+    elif (TIDE_MODEL == 'Gr1km-v2'):
+        grid_file = os.path.join(tide_dir,'greenlandTMD_v2','grid_Greenland8.v2')
+        model_file = os.path.join(tide_dir,'greenlandTMD_v2','h_Greenland8.v2')
+        reference = 'https://www.esr.org/research/polar-tide-models/'
+        variable = 'tide_ocean'
+        long_name = "Ocean Tide"
+        description = ("Ocean Tides including diurnal and semi-diurnal "
+            "(harmonic analysis), and longer period tides (dynamic and "
+            "self-consistent equilibrium).")
+        model_format = 'OTIS'
+        EPSG = '3413'
+        TYPE = 'z'
     elif (TIDE_MODEL == 'GOT4.7'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
@@ -536,23 +553,25 @@ def compute_tides_ICESat2(tide_dir, INPUT_FILE, TIDE_MODEL=None, METHOD='spline'
         if model_format in ('OTIS','ATLAS'):
             amp,ph,D,c = extract_tidal_constants(val['longitude'],
                 val['latitude'], grid_file, model_file, EPSG, TYPE=TYPE,
-                METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
+                METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF,
+                GRID=model_format)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'netcdf'):
             amp,ph,D,c = extract_netcdf_constants(val['longitude'],
                 val['latitude'], grid_file, model_file, TYPE=TYPE, METHOD=METHOD,
-                EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
+                EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'GOT'):
             amp,ph,c = extract_GOT_constants(val['longitude'], val['latitude'],
-                model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE,
-                GZIP=GZIP)
+                model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+                CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
             deltat = calc_delta_time(delta_file, tide_time)
         elif (model_format == 'FES'):
             amp,ph = extract_FES_constants(val['longitude'], val['latitude'],
                 model_file, TYPE=TYPE, VERSION=TIDE_MODEL, METHOD=METHOD,
-                EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
+                EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=SCALE,
+                GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
             deltat = calc_delta_time(delta_file, tide_time)
 
@@ -881,7 +900,7 @@ def main():
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
         'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
-        'AODTM-5','AOTIM-5','AOTIM-5-2018',
+        'AODTM-5','AOTIM-5','AOTIM-5-2018','Gr1km-v2',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
         'FES2014','FES2014_load')
     parser.add_argument('--tide','-T',
@@ -897,6 +916,11 @@ def main():
     parser.add_argument('--extrapolate','-E',
         default=False, action='store_true',
         help='Extrapolate with nearest-neighbors')
+    #-- extrapolation cutoff in kilometers
+    #-- set to inf to extrapolate over all points
+    parser.add_argument('--cutoff','-c',
+        type=np.float64, default=10.0,
+        help='Extrapolation cutoff in kilometers')
     #-- verbosity settings
     #-- verbose will output information about each output file
     parser.add_argument('--verbose','-V',
@@ -912,7 +936,7 @@ def main():
     for FILE in args.infile:
         compute_tides_ICESat2(args.directory, FILE, TIDE_MODEL=args.tide,
             METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
-            VERBOSE=args.verbose, MODE=args.mode)
+            CUTOFF=args.cutoff, VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
