@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tides_icebridge_data.py
-Written by Tyler Sutterley (03/2021)
+Written by Tyler Sutterley (06/2021)
 Calculates tidal elevations for correcting Operation IceBridge elevation data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -31,6 +31,7 @@ COMMAND LINE OPTIONS:
         AODTM-5
         AOTIM-5
         AOTIM-5-2018
+        Gr1km-v2
         GOT4.7
         GOT4.7_load
         GOT4.8
@@ -45,6 +46,8 @@ COMMAND LINE OPTIONS:
         nearest
         bilinear
     -E X, --extrapolate X: Extrapolate with nearest-neighbors
+    -c X, --cutoff X: Extrapolation cutoff in kilometers
+        set to inf to extrapolate for all points
     -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
@@ -63,7 +66,7 @@ PYTHON DEPENDENCIES:
 
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
-    utilities: download and management utilities for syncing files
+    utilities.py: download and management utilities for syncing files
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
@@ -80,6 +83,9 @@ PROGRAM DEPENDENCIES:
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 06/2021: added new Gr1km-v2 1km Greenland model from ESR
+    Updated 05/2021: added option for extrapolation cutoff in kilometers
+        modified import of ATM1b QFIT reader
     Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
         simplified netcdf inputs to be similar to binary OTIS read program
         replaced numpy bool/int to prevent deprecation warnings
@@ -113,7 +119,7 @@ import argparse
 import numpy as np
 import pyTMD.time
 from pyTMD.utilities import get_data_path
-from read_ATM1b_QFIT_binary.read_ATM1b_QFIT_binary import read_ATM1b_QFIT_binary
+import read_ATM1b_QFIT_binary.read_ATM1b_QFIT_binary as ATM1b
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.infer_minor_corrections import infer_minor_corrections
 from pyTMD.predict_tide_drift import predict_tide_drift
@@ -133,11 +139,11 @@ def file_length(input_file, input_subsetter, HDF5=False, QFIT=False):
             file_lines, = fileID[HDF5].shape
     elif QFIT:
         #-- read the size of a QFIT binary file
-        file_lines = read_ATM1b_QFIT_binary.ATM1b_QFIT_shape(input_file)
+        file_lines = ATM1b.ATM1b_QFIT_shape(input_file)
     else:
         #-- read the input file, split at lines and remove all commented lines
         with open(input_file,'r') as f:
-            i = [i for i in f.read().splitlines() if re.match(r'^(?!#)',i)]
+            i = [i for i in f.read().splitlines() if re.match(r'^(?!\#)',i)]
         file_lines = len(i)
     #-- return the number of lines
     return file_lines
@@ -167,7 +173,7 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         #-- read the input file, split at lines and remove all commented lines
         with open(input_file,'r') as f:
             file_contents = [i for i in f.read().splitlines() if
-                re.match(r'^(?!#)',i)]
+                re.match(r'^(?!\#)',i)]
         #-- number of lines of data within file
         file_lines = file_length(input_file,input_subsetter)
         #-- create output variables with length equal to the number of lines
@@ -190,7 +196,7 @@ def read_ATM_qfit_file(input_file, input_subsetter):
     #-- Version 1 of ATM QFIT files (binary)
     elif (SFX == 'qi'):
         #-- read input QFIT data file and subset if specified
-        fid,h = read_ATM1b_QFIT_binary(input_file)
+        fid,h = ATM1b.read_ATM1b_QFIT_binary(input_file)
         #-- number of lines of data within file
         file_lines = file_length(input_file,input_subsetter,QFIT=True)
         ATM_L1b_input['lat'] = fid['latitude'][:]
@@ -277,7 +283,7 @@ def read_ATM_icessn_file(input_file, input_subsetter):
     #-- read the input file, split at lines and remove all commented lines
     with open(input_file,'r') as f:
         file_contents = [i for i in f.read().splitlines() if
-            re.match(r'^(?!#)',i)]
+            re.match(r'^(?!\#)',i)]
     #-- number of lines of data within file
     file_lines = file_length(input_file,input_subsetter)
     #-- output python dictionary with variables
@@ -417,8 +423,8 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
 
 #-- PURPOSE: read Operation IceBridge data from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
-def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
-    METHOD='spline', EXTRAPOLATE=False, VERBOSE=False, MODE=0o775):
+def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL, METHOD='spline',
+    EXTRAPOLATE=False, CUTOFF=None, VERBOSE=False, MODE=0o775):
 
     #-- extract file name and subsetter indices lists
     match_object = re.match(r'(.*?)(\[(.*?)\])?$',arg)
@@ -610,6 +616,15 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
         model_format = 'OTIS'
         EPSG = 'PSNorth'
         TYPE = 'z'
+    elif (TIDE_MODEL == 'Gr1km-v2'):
+        grid_file = os.path.join(tide_dir,'greenlandTMD_v2','grid_Greenland8.v2')
+        model_file = os.path.join(tide_dir,'greenlandTMD_v2','h_Greenland8.v2')
+        reference = 'https://www.esr.org/research/polar-tide-models/'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
+        model_format = 'OTIS'
+        EPSG = '3413'
+        TYPE = 'z'
     elif (TIDE_MODEL == 'GOT4.7'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
@@ -801,23 +816,25 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(dinput['lon'], dinput['lat'],
             grid_file, model_file, EPSG, TYPE=TYPE, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, GRID=model_format)
         deltat = np.zeros_like(t)
     elif model_format in ('netcdf'):
         amp,ph,D,c = extract_netcdf_constants(dinput['lon'], dinput['lat'],
             grid_file, model_file, TYPE=TYPE, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=SCALE,
+            GZIP=GZIP)
         deltat = np.zeros_like(t)
     elif (model_format == 'GOT'):
         amp,ph,c = extract_GOT_constants(dinput['lon'], dinput['lat'],
             model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
-            SCALE=SCALE, GZIP=GZIP)
+            CUTOFF=CUTOFF, SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file, t)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(dinput['lon'], dinput['lat'],
             model_file, TYPE=TYPE, VERSION=TIDE_MODEL, METHOD=METHOD,
-            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
+            EXTRAPOLATE=EXTRAPOLATE, CUTOFF=CUTOFF, SCALE=SCALE,
+            GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
         deltat = calc_delta_time(delta_file, t)
 
@@ -952,7 +969,7 @@ def main():
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
         'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
-        'AODTM-5','AOTIM-5','AOTIM-5-2018',
+        'AODTM-5','AOTIM-5','AOTIM-5-2018','Gr1km-v2',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
         'FES2014','FES2014_load')
     parser.add_argument('--tide','-T',
@@ -968,6 +985,11 @@ def main():
     parser.add_argument('--extrapolate','-E',
         default=False, action='store_true',
         help='Extrapolate with nearest-neighbors')
+    #-- extrapolation cutoff in kilometers
+    #-- set to inf to extrapolate over all points
+    parser.add_argument('--cutoff','-c',
+        type=np.float64, default=10.0,
+        help='Extrapolation cutoff in kilometers')
     #-- verbosity settings
     #-- verbose will output information about each output file
     parser.add_argument('--verbose','-V',
@@ -983,7 +1005,7 @@ def main():
     for arg in args.infile:
         compute_tides_icebridge_data(args.directory, arg, TIDE_MODEL=args.tide,
             METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
-            VERBOSE=args.verbose, MODE=args.mode)
+            CUTOFF=args.cutoff, VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
